var Future = require("fibers/future");
var child_process = require("child_process");
var util = require("util");

var ChildProcess = (function () {
    function ChildProcess($logger) {
        this.$logger = $logger;
        this._execFile = Future.wrap(function (command, args, callback) {
            return child_process.execFile(command, args, callback);
        });
    }
    ChildProcess.prototype.exec = function (command) {
        var _this = this;
        var future = new Future();
        child_process.exec(command, function (error, stdout, stderr) {
            _this.$logger.trace("Exec %s \n stdout: %s \n stderr: %s", command, stdout.toString(), stderr.toString());

            if (error) {
                future.throw(error);
            } else {
                future.return(stdout);
            }
        });

        return future;
    };

    ChildProcess.prototype.execFile = function (command, args) {
        this.$logger.debug("execFile: %s %s", command, args.join(" "));
        return this._execFile(command, args);
    };

    ChildProcess.prototype.spawn = function (command, args, options) {
        this.$logger.debug("spawn: %s %s", command, args.join(" "));
        return child_process.spawn(command, args, options);
    };

    ChildProcess.prototype.spawnFromEvent = function (command, args, event, options, spawnFromEventOptions) {
        var future = new Future();
        var childProcess = this.spawn(command, args, options);

        var capturedOut = '';
        var capturedErr = '';

        if (childProcess.stdout) {
            childProcess.stdout.on("data", function (data) {
                capturedOut += data;
            });
        }

        if (childProcess.stderr) {
            childProcess.stderr.on("data", function (data) {
                capturedErr += data;
            });
        }

        childProcess.on(event, function (arg) {
            var exitCode = typeof arg == 'number' ? arg : arg && arg.code;
            var result = {
                stdout: capturedOut,
                stderr: capturedErr,
                exitCode: exitCode
            };

            if (spawnFromEventOptions && spawnFromEventOptions.throwError === false) {
                future.return(result);
            } else {
                if (exitCode === 0) {
                    future.return(result);
                } else {
                    var errorMessage = util.format('Command %s failed with exit code %d', command, exitCode);
                    if (capturedErr) {
                        errorMessage += util.format(' Error output: \n %s', capturedErr);
                    }
                    future.throw(errorMessage);
                }
            }
        });

        return future;
    };
    return ChildProcess;
})();
exports.ChildProcess = ChildProcess;
$injector.register("childProcess", ChildProcess);
