"use strict";
var util = require("util");

var HelpCommand = (function () {
    function HelpCommand($logger, $injector, $errors, $fs, $staticConfig) {
        this.$logger = $logger;
        this.$injector = $injector;
        this.$errors = $errors;
        this.$fs = $fs;
        this.$staticConfig = $staticConfig;
        this.enableHooks = false;
    }
    HelpCommand.prototype.execute = function (args) {
        var _this = this;
        return (function () {
            var topic = (args[0] || "").toLowerCase();
            if (topic === "help") {
                topic = "";
            }

            if (args[1]) {
                topic = util.format("%s|%s", args[0], args[1]);
            }

            var helpContent = _this.$fs.readText(_this.$staticConfig.helpTextPath).wait();

            var pattern = util.format("--\\[%s\\]--((.|[\\r\\n])+?)--\\[/\\]--", RegExp.escape(topic));
            var regex = new RegExp(pattern);

            var match = regex.exec(helpContent);
            if (match) {
                var helpText = match[1].trim();

                var substitutionPoint;
                while (substitutionPoint = helpText.match(_this.$injector.dynamicCallRegex)) {
                    _this.$logger.trace(substitutionPoint);
                    var data = _this.$injector.dynamicCall(substitutionPoint[0]).wait();

                    var pointStart = substitutionPoint.index;
                    var pointEnd = pointStart + substitutionPoint[0].length;
                    helpText = helpText.substr(0, pointStart) + data + helpText.substr(pointEnd);
                }

                _this.$logger.out(helpText);
            } else {
                _this.$errors.fail({ formatStr: "Unknown help topic '%s'", suppressCommandHelp: true }, topic);
            }
        }).future()();
    };
    return HelpCommand;
})();
exports.HelpCommand = HelpCommand;
$injector.registerCommand(["help", "/?"], HelpCommand);
