var Fiber = require("fibers");
var Future = require("fibers/future");

var queue = require("./queue");

var options = require("./options");

var CommandDispatcher = (function () {
    function CommandDispatcher($logger, $cancellation, $commandsService, $staticConfig) {
        this.$logger = $logger;
        this.$cancellation = $cancellation;
        this.$commandsService = $commandsService;
        this.$staticConfig = $staticConfig;
    }
    CommandDispatcher.prototype.dispatchCommand = function () {
        var _this = this;
        return (function () {
            if (options.version) {
                _this.$logger.out(_this.$staticConfig.version);
                return;
            }

            var commandName = _this.getCommandName();
            var commandArguments = _this.getCommandArguments();

            if (options.help) {
                commandArguments.unshift(commandName);
                commandName = "help";
            }

            _this.$cancellation.begin("cli").wait();

            _this.$commandsService.tryExecuteCommand(commandName, commandArguments).wait();
        }).future()();
    };

    CommandDispatcher.prototype.completeCommand = function (commandsWithPlatformArgument, platforms, propSchema) {
        return this.$commandsService.completeCommand(commandsWithPlatformArgument, platforms, propSchema);
    };

    CommandDispatcher.prototype.getCommandName = function () {
        var remaining = options._;
        if (remaining.length > 0) {
            return remaining[0].toLowerCase();
        }
        return "help";
    };

    CommandDispatcher.prototype.getCommandArguments = function () {
        var remaining = options._;
        if (remaining.length > 1) {
            remaining = remaining.slice(1);
            remaining.forEach(function (item, idx, array) {
                if (typeof item === "number") {
                    array[idx] = item.toString();
                }
            });
            return remaining;
        }
        return [];
    };
    return CommandDispatcher;
})();
exports.CommandDispatcher = CommandDispatcher;
$injector.register("commandDispatcher", CommandDispatcher);

var FutureDispatcher = (function () {
    function FutureDispatcher($errors) {
        this.$errors = $errors;
    }
    FutureDispatcher.prototype.run = function () {
        if (this.actions) {
            this.$errors.fail("You cannot run a running future dispatcher.");
        }
        this.actions = new queue.Queue();

        while (true) {
            var action = this.actions.dequeue().wait();
            action().wait();
        }
    };

    FutureDispatcher.prototype.dispatch = function (action) {
        this.actions.enqueue(action);
    };

    FutureDispatcher.prototype.runMainFiber = function () {
        var fiber = Fiber(function () {
            var commandDispatcher = $injector.resolve("commandDispatcher");

            if (process.argv[2] === "completion") {
                commandDispatcher.completeCommand(["add", "remove", "prepare", "build", "deploy", "emulate", "run"], ["android", "ios"]).wait();
            } else {
                commandDispatcher.dispatchCommand().wait();
            }

            $injector.dispose();
            Future.assertNoFutureLeftBehind();
        });
        global.__main_fiber__ = fiber;
        fiber.run();
    };
    return FutureDispatcher;
})();
$injector.register("dispatcher", FutureDispatcher, false);
