"use strict";
var util = require("util");
var Future = require("fibers/future");
var hostInfo = require("../../../common/host-info");
var MobileHelper = require("./../mobile-helper");

var IosEmulatorServices = (function () {
    function IosEmulatorServices($logger, $emulatorSettingsService, $errors, $childProcess) {
        this.$logger = $logger;
        this.$emulatorSettingsService = $emulatorSettingsService;
        this.$errors = $errors;
        this.$childProcess = $childProcess;
    }
    IosEmulatorServices.prototype.checkAvailability = function (dependsOnProject) {
        var _this = this;
        if (typeof dependsOnProject === "undefined") { dependsOnProject = true; }
        return (function () {
            if (!hostInfo.isDarwin()) {
                _this.$errors.fail("iOS Simulator is available only on Mac OS X.");
            }

            try  {
                _this.$childProcess.exec(util.format("which ", IosEmulatorServices.SimulatorLauncher)).wait();
            } catch (err) {
                _this.$errors.fail("Unable to find ios-sim. Run `npm install -g ios-sim` to install it.");
            }

            var platform = MobileHelper.DevicePlatforms[0 /* iOS */];
            if (dependsOnProject && !_this.$emulatorSettingsService.canStart(platform).wait()) {
                _this.$errors.fail("The current project does not target iOS and cannot be run in the iOS Simulator.");
            }
        }).future()();
    };

    IosEmulatorServices.prototype.startEmulator = function (app, emulatorOptions) {
        var _this = this;
        return (function () {
            _this.killLaunchdSim().wait();
            _this.startEmulatorCore(app, emulatorOptions);
        }).future()();
    };

    IosEmulatorServices.prototype.killLaunchdSim = function () {
        this.$logger.info("Cleaning up before starting the iOS Simulator");

        var future = new Future();
        var killAllProc = this.$childProcess.spawn("killall", ["launchd_sim"]);
        killAllProc.on("close", function (code) {
            future.return();
        });
        return future;
    };

    IosEmulatorServices.prototype.startEmulatorCore = function (app, emulatorOptions) {
        this.$logger.info("Starting iOS Simulator");

        var opts = [
            "launch", app,
            "--exit"
        ];

        if (emulatorOptions) {
            if (emulatorOptions.stderrFilePath) {
                opts = opts.concat("--stderr", emulatorOptions.stderrFilePath);
            }
            if (emulatorOptions.stdoutFilePath) {
                opts = opts.concat("--stdout", emulatorOptions.stdoutFilePath);
            }
            if (emulatorOptions.deviceFamily) {
                opts = opts.concat("--family", emulatorOptions.deviceFamily);
            }
        }
        this.$childProcess.spawn(IosEmulatorServices.SimulatorLauncher, opts, { stdio: ["ignore", "ignore", "ignore"], detached: true }).unref();
    };

    IosEmulatorServices.SimulatorLauncher = "ios-sim";
    return IosEmulatorServices;
})();
$injector.register("iOSEmulatorServices", IosEmulatorServices);
