"use strict";
var helpers = require("../helpers");

(function (DevicePlatforms) {
    DevicePlatforms[DevicePlatforms["iOS"] = 0] = "iOS";
    DevicePlatforms[DevicePlatforms["Android"] = 1] = "Android";
    DevicePlatforms[DevicePlatforms["WP8"] = 2] = "WP8";
})(exports.DevicePlatforms || (exports.DevicePlatforms = {}));
var DevicePlatforms = exports.DevicePlatforms;

exports.platformCapabilities = {
    iOS: {
        wirelessDeploy: true,
        cableDeploy: true,
        companion: true,
        hostPlatformsForDeploy: ["win32", "darwin"]
    },
    Android: {
        wirelessDeploy: true,
        cableDeploy: true,
        companion: true,
        hostPlatformsForDeploy: ["win32", "darwin", "linux"]
    },
    WP8: {
        wirelessDeploy: true,
        cableDeploy: false,
        companion: false,
        hostPlatformsForDeploy: ["win32"]
    }
};

exports.PlatformNames = Object.keys(exports.platformCapabilities);

var LocalToDevicePathData = (function () {
    function LocalToDevicePathData(localPath, devicePath, relativeToProjectBasePath) {
        this.localPath = localPath;
        this.devicePath = devicePath;
        this.relativeToProjectBasePath = relativeToProjectBasePath;
    }
    LocalToDevicePathData.prototype.getLocalPath = function () {
        return this.localPath;
    };
    LocalToDevicePathData.prototype.getDevicePath = function () {
        return this.devicePath;
    };
    LocalToDevicePathData.prototype.getRelativeToProjectBasePath = function () {
        return this.relativeToProjectBasePath;
    };
    return LocalToDevicePathData;
})();
exports.LocalToDevicePathData = LocalToDevicePathData;

function isAndroidPlatform(platform) {
    return DevicePlatforms[1 /* Android */].toLowerCase() === platform.toLowerCase();
}
exports.isAndroidPlatform = isAndroidPlatform;

function isiOSPlatform(platform) {
    return DevicePlatforms[0 /* iOS */].toLowerCase() === platform.toLowerCase();
}
exports.isiOSPlatform = isiOSPlatform;

function isWP8Platform(platform) {
    return DevicePlatforms[2 /* WP8 */].toLowerCase() === platform.toLowerCase();
}
exports.isWP8Platform = isWP8Platform;

function normalizePlatformName(platform) {
    if (exports.isAndroidPlatform(platform)) {
        return "Android";
    } else if (exports.isiOSPlatform(platform)) {
        return "iOS";
    } else if (exports.isWP8Platform(platform)) {
        return "WP8";
    }

    return undefined;
}
exports.normalizePlatformName = normalizePlatformName;

function isPlatformSupported(platform) {
    var platformName = exports.normalizePlatformName(platform);
    return _.contains(exports.platformCapabilities[platformName].hostPlatformsForDeploy, process.platform);
}
exports.isPlatformSupported = isPlatformSupported;

function generateWP8GUID() {
    return helpers.createGUID();
}
exports.generateWP8GUID = generateWP8GUID;

function validatePlatformName(platform, $errors) {
    if (!platform) {
        $errors.fail("No device platform specified.");
    }

    var normalizedPlatform = exports.normalizePlatformName(platform);
    if (!normalizedPlatform) {
        $errors.fail("'%s' is not a valid device platform. Valid platforms are %s.", platform, helpers.formatListOfNames(Object.keys(exports.platformCapabilities)));
    }
    return normalizedPlatform;
}
exports.validatePlatformName = validatePlatformName;
