var path = require("path");
var options = require("./options");
var util = require("util");

var ProjectHelper = (function () {
    function ProjectHelper($logger, $fs, $staticConfig) {
        this.$logger = $logger;
        this.$fs = $fs;
        this.$staticConfig = $staticConfig;
        this.cachedProjectDir = "";
    }
    Object.defineProperty(ProjectHelper.prototype, "projectDir", {
        get: function () {
            if (this.cachedProjectDir !== "") {
                return this.cachedProjectDir;
            }
            this.cachedProjectDir = null;

            var projectDir = path.resolve(options.path || ".");
            while (true) {
                this.$logger.trace("Looking for project in '%s'", projectDir);

                if (this.$fs.exists(path.join(projectDir, this.$staticConfig.PROJECT_FILE_NAME)).wait()) {
                    this.$logger.debug("Project directory is '%s'.", projectDir);
                    this.cachedProjectDir = projectDir;
                    break;
                }

                var dir = path.dirname(projectDir);
                if (dir === projectDir) {
                    this.$logger.debug("No project found at or above '%s'.", path.resolve("."));
                    break;
                }
                projectDir = dir;
            }

            return this.cachedProjectDir;
        },
        enumerable: true,
        configurable: true
    });

    ProjectHelper.prototype.generateDefaultAppId = function (appName, baseAppId) {
        var sanitizedName = _.filter(appName.split(""), function (c) {
            return /[a-zA-Z0-9]/.test(c);
        }).join("");
        if (sanitizedName) {
            if (/^\d+$/.test(sanitizedName)) {
                sanitizedName = "the" + sanitizedName;
            }
        } else {
            sanitizedName = "the";
        }

        return util.format("%s.%s", baseAppId, sanitizedName);
    };
    return ProjectHelper;
})();
exports.ProjectHelper = ProjectHelper;
$injector.register("projectHelper", ProjectHelper);
