var Future = require("fibers/future");
var prompt = require("prompt");
var helpers = require("./helpers");
var readline = require("readline");
var MuteStream = require("mute-stream");

var Prompter = (function () {
    function Prompter() {
        prompt.message = "";
        prompt.delimiter = ":";
        prompt.colors = false;
        prompt.isDefaultValueEditable = true;

        if (helpers.isInteractive()) {
            process.stdin.setRawMode(true);

            var mutestream = new MuteStream();
            mutestream.pipe(process.stdout);
            mutestream.mute();

            this.ctrlcReader = readline.createInterface({
                input: process.stdin,
                output: mutestream
            });

            this.ctrlcReader.on("SIGINT", function () {
                return process.exit();
            });
        }
    }
    Prompter.prototype.start = function () {
        prompt.start();
    };

    Prompter.prototype.dispose = function () {
        if (this.ctrlcReader) {
            this.ctrlcReader.close();
        }
    };

    Prompter.prototype.get = function (schema) {
        var future = new Future;
        prompt.get(schema, function (err, result) {
            if (err) {
                console.log("");
                err.suppressCommandHelp = true;
                future.throw(err);
            } else {
                future.return(result);
            }
        });
        return future;
    };

    Prompter.prototype.getPassword = function (prompt, options) {
        var _this = this;
        return (function () {
            var schema = {
                properties: {
                    password: {
                        description: prompt,
                        type: "string",
                        hidden: true,
                        required: !(options && options.allowEmpty),
                        message: "Password must be non-empty."
                    }
                }
            };

            var result = _this.get(schema).wait();
            return result.password;
        }).future()();
    };

    Prompter.prototype.confirm = function (prompt, defaultAction) {
        var _this = this;
        return (function () {
            var schema = {
                properties: {
                    prompt: {
                        description: prompt + " (y/n)",
                        type: "string",
                        required: true,
                        default: defaultAction,
                        message: "Enter 'y' (for yes) or 'n' (for no).",
                        conform: function (value) {
                            return /^[yn]$/i.test(value);
                        }
                    }
                }
            };

            var result = _this.get(schema).wait();
            return result.prompt.toLowerCase() === "y";
        }).future()();
    };

    Prompter.prototype.history = function (name) {
        return prompt.history(name);
    };

    Prompter.prototype.override = function (object) {
        prompt.override = object;
    };
    return Prompter;
})();
exports.Prompter = Prompter;
$injector.register("prompter", Prompter);
