"use strict";
var osenv = require("osenv");
var path = require("path");
var util = require("util");

var AutoCompletionService = (function () {
    function AutoCompletionService($fs, $childProcess, $logger, $staticConfig) {
        this.$fs = $fs;
        this.$childProcess = $childProcess;
        this.$logger = $logger;
        this.$staticConfig = $staticConfig;
        this.scriptsOk = true;
        this.disableAnalytics = true;
    }
    AutoCompletionService.prototype.enableAutoCompletion = function () {
        var _this = this;
        return (function () {
            _this.updateShellScript(".bashrc").wait();
            _this.updateShellScript(".bash_profile").wait();
            _this.updateShellScript(".zshrc").wait();

            if (_this.scriptsOk) {
                _this.$logger.out("Restart your shell to enable command auto-completion.");
            }
        }).future()();
    };

    AutoCompletionService.prototype.updateShellScript = function (fileName) {
        var _this = this;
        return (function () {
            try  {
                _this.updateShellScriptCore(fileName).wait();
            } catch (err) {
                _this.$logger.out("Failed to update %s. Auto-completion may not work. ", fileName);
                _this.$logger.out(err);
                _this.scriptsOk = false;
            }
        }).future()();
    };

    AutoCompletionService.prototype.updateShellScriptCore = function (fileName) {
        var _this = this;
        return (function () {
            var filePath = _this.getHomePath(fileName);

            var doUpdate = true;
            if (_this.$fs.exists(filePath).wait()) {
                var contents = _this.$fs.readText(filePath).wait();
                var regExp = new RegExp(util.format("%s\\s+completion\\s+--\\s+", _this.$staticConfig.CLIENT_NAME));
                var matchCondition = contents.match(regExp);
                if (_this.$staticConfig.CLIENT_NAME_ALIAS) {
                    matchCondition = matchCondition || contents.match(new RegExp(util.format("%s\\s+completion\\s+--\\s+", _this.$staticConfig.CLIENT_NAME_ALIAS.toLowerCase())));
                }

                if (matchCondition) {
                    doUpdate = false;
                }
            }

            if (doUpdate) {
                _this.$childProcess.exec(_this.$staticConfig.CLIENT_NAME.toLowerCase() + " completion >> " + filePath).wait();
            }
        }).future()();
    };

    AutoCompletionService.prototype.getHomePath = function (fileName) {
        return path.join(osenv.home(), fileName);
    };
    return AutoCompletionService;
})();
exports.AutoCompletionService = AutoCompletionService;
$injector.register("autoCompletionService", AutoCompletionService);
