var path = require("path");
var assert = require("assert");

var ProjectDataService = (function () {
    function ProjectDataService($fs, $staticConfig) {
        this.$fs = $fs;
        this.$staticConfig = $staticConfig;
    }
    ProjectDataService.prototype.initialize = function (projectDir) {
        if (!this.projectFileName) {
            this.projectFileName = path.join(projectDir, this.$staticConfig.PROJECT_FILE_NAME);
        }
    };

    ProjectDataService.prototype.getValue = function (propertyName) {
        var _this = this;
        return (function () {
            _this.loadProjectFile().wait();
            return _this.projectData ? _this.projectData[propertyName] : null;
        }).future()();
    };

    ProjectDataService.prototype.setValue = function (key, value) {
        var _this = this;
        return (function () {
            _this.loadProjectFile().wait();
            _this.projectData[key] = value;
            _this.$fs.writeJson(_this.projectFileName, _this.projectData, "\t").wait();
        }).future()();
    };

    ProjectDataService.prototype.loadProjectFile = function () {
        var _this = this;
        return (function () {
            assert.ok(_this.projectFileName, "Initialize method of projectDataService is not called");

            if (!_this.projectData) {
                if (!_this.$fs.exists(_this.projectFileName).wait()) {
                    _this.$fs.writeFile(_this.projectFileName, null).wait();
                }

                _this.projectData = _this.$fs.readJson(_this.projectFileName).wait() || Object.create(null);
            }
        }).future()();
    };
    return ProjectDataService;
})();
exports.ProjectDataService = ProjectDataService;
$injector.register("projectDataService", ProjectDataService);
